/*--------------------------------------------------------------------------*/
//	Test program for DirectX Graphics 8
//
//	by : Ngan Lo '2002
//
//	email : nganlo8@email.com
/*--------------------------------------------------------------------------*/

// Includes -------------------------------------------------------
#define WIN32_LEAN_AND_MEAN
#define INITGUID

#include <windows.h>
#include <mmsystem.h>
#include <time.h>
#include <d3d8.h>
#include <d3dx8.h>
#include <D3dx8math.h>

#define WINDOW_WIDTH 640
#define WINDOW_HEIGHT 480
#define WINDOW_CLASS_NAME "NGANLOCLASS"
#define WINDOWS_TITLE	"Ngan Lo's GAME"

#define DX8RELEASE(x) if (x) { x->lpVtbl->Release(x); x=NULL; }
#define KEY_DOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEY_UP(vk_code)   ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)

#define SCRSHOT_NAME	"sshot.bmp"

// Vertex type to use in this demo
typedef struct  { FLOAT x, y, z; DWORD color; FLOAT u, v; } CUSTOMVERTEX;
#define D3DFVF_CUSTOMVERTEX ( D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1 )

// Prototypes -----------------------------------------------------
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hprevinstance, LPSTR lpcmdline, int ncmdshow);

//-- Game main modules
int Game_Init(void);
int Game_Shutdown(void);
int Game_Main(void);

// Global -------------------------------------------------
IDirect3D8 *g_pD3D=NULL;
IDirect3DDevice8 *g_pd3dDevice = NULL;
LPDIRECT3DVERTEXBUFFER8 g_pVB = NULL;
LPDIRECT3DTEXTURE8	pTex = NULL;

HWND		g_hWnd;
HINSTANCE	g_Hinst;
BOOL		g_bActive;
BOOL		g_bReady;
BOOL		g_bWindowed=TRUE;

D3DCOLOR color;
char tmp[40];

// CALLBACK  WindowProc -----------------------------------
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	PAINTSTRUCT	ps;
	HDC		hdc;

	switch(msg)
	{
	case WM_CREATE:
        {
		// do initialization stuff here but NOT DirectX
		return(0);
		} break;

	case WM_ACTIVATE:
		{
            // Pause if minimized
            g_bActive = !((BOOL)HIWORD(wparam));
            return (0);
		} break;
	case WM_PAINT:
        {
         	// start painting
			BeginPaint(hwnd,&ps);
			ValidateRect( hwnd, NULL );

         	// end painting
			EndPaint(hwnd,&ps);
			return(0);
        } break;

	case WM_DESTROY:
		{
		// kill the application
		Game_Shutdown();
		PostQuitMessage(0);
		return(0);
		} break;
	case WM_SETCURSOR:
		{
			if (g_bActive && !g_bWindowed)
			{
				SetCursor(NULL);
				return TRUE;
			}
		} break;
	case WM_SIZE:
		{
		// Check if we are resizing our window...
		if (SIZE_MAXHIDE==wparam || SIZE_MINIMIZED==wparam)
			g_bActive = FALSE;
		else
			g_bActive = TRUE;
		} break;
	case WM_KEYDOWN:
		switch(wparam)
		{
			// Emergency Exit
			case VK_F12:
				Game_Shutdown();
				PostMessage(hwnd,WM_DESTROY,0,0);
			break;
		} break;
	case MM_MCINOTIFY:  //midi replay

        break;
	default:break;
    }
	return (DefWindowProc(hwnd, msg, wparam, lparam));
}

/*--------------------------------------------------------------------------*/
// WinMain
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hprevinstance, LPSTR lpcmdline, int ncmdshow)
{
	WNDCLASS winclass;
	HWND	 hwnd;
	MSG		 msg;
	RECT rc;

	CoInitialize(NULL);

	winclass.style			= CS_HREDRAW | CS_VREDRAW;
	winclass.lpfnWndProc	= WindowProc;
	winclass.cbClsExtra		= 0;
	winclass.cbWndExtra		= 0;
	winclass.hInstance		= hinstance;
	winclass.hIcon			= LoadIcon(hinstance, IDI_APPLICATION);
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
	winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName	= NULL;
	winclass.lpszClassName	= WINDOW_CLASS_NAME;

		// register the window class
		if (!RegisterClass(&winclass))
			return(0);

		// Calculate size of window based on desired client window size
		rc.left = 0;
		rc.top = 0;
		rc.right = WINDOW_WIDTH;
		rc.bottom = WINDOW_HEIGHT;
		//AdjustWindowRect( &rc, WS_OVERLAPPEDWINDOW, FALSE );

		hwnd = CreateWindow(WINDOW_CLASS_NAME,
						WINDOWS_TITLE,
						WS_OVERLAPPED|WS_SYSMENU,
						CW_USEDEFAULT, CW_USEDEFAULT,
						rc.right-rc.left, rc.bottom-rc.top,
						NULL, NULL, hinstance, NULL);

	ShowWindow(hwnd,ncmdshow);
	UpdateWindow(hwnd);

	// Save the global hWnd & hInst values
	g_hWnd = hwnd;
	g_Hinst = hinstance;

	//InvalidateRect( g_hWnd, NULL, TRUE );

	Game_Init();

	// Enter main event loop
	while(1)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			if (msg.message == WM_QUIT)
				break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else if (g_bActive)
		{
			Game_Main();
		}
		else WaitMessage();
	}
	Game_Shutdown();

	return(msg.wParam);
}

// Game initialization module here  -----------------------

void InitVector(D3DVECTOR& v, const float fx, const float fy, const float fz)
{
	v.x=fx;
	v.y=fy;
	v.z=fz;
}

void SetVertex(CUSTOMVERTEX& v,float x,float y,float z,DWORD color,float tu,float tv)
{
	v.x=x;
	v.y=y;
	v.z=z;
	v.color=color;
	v.u=tu;
	v.v=tv;
}

int Game_Init(void)
{
	HRESULT hr;
	srand( (unsigned)time( NULL ) );

	g_pD3D = Direct3DCreate8 (D3D_SDK_VERSION);

	D3DDISPLAYMODE d3ddm;
	hr=g_pD3D->lpVtbl->GetAdapterDisplayMode (g_pD3D,D3DADAPTER_DEFAULT, &d3ddm);
	if (hr!=D3D_OK)
		MessageBox(g_hWnd,"GetAdapterDisplayMode failed","Error",MB_OK);

	D3DPRESENT_PARAMETERS	d3dpp;
	ZeroMemory (&d3dpp, sizeof(d3dpp));
	d3dpp.Windowed = g_bWindowed;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = d3ddm.Format;;

	hr=g_pD3D->lpVtbl->CreateDevice(g_pD3D,D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,g_hWnd,D3DCREATE_SOFTWARE_VERTEXPROCESSING,&d3dpp,&g_pd3dDevice );
	if (hr!=D3D_OK)
		MessageBox(g_hWnd,"CreateDevice failed","Error",MB_OK);

	// use d3dx8 to load texture.
	hr=D3DXCreateTextureFromFileEx( g_pd3dDevice, "carpet.tga", D3DX_DEFAULT, D3DX_DEFAULT, 0, 0, D3DFMT_UNKNOWN, D3DPOOL_MANAGED,
								D3DX_FILTER_LINEAR, D3DX_FILTER_LINEAR, 0xFF000000, NULL, NULL, &pTex );
	if (hr!=D3D_OK)
		MessageBox(g_hWnd,"D3DXCreateTextureFromFileEX failed","Error",MB_OK);
	CUSTOMVERTEX g_Vertices[4];

	SetVertex(g_Vertices[0], -1.0f, -1.0f, 0.0f, 0xffffffff, 0,1);
	SetVertex(g_Vertices[1], -1.0f,  1.0f, 0.0f, 0xffffffff, 0,0);
	SetVertex(g_Vertices[2],  1.0f, -1.0f, 0.0f, 0xffffffff, 1,1);
	SetVertex(g_Vertices[3],  1.0f,  1.0f, 0.0f, 0xffffffff, 1,0);

	g_pd3dDevice->lpVtbl->CreateVertexBuffer (g_pd3dDevice, 6*sizeof(CUSTOMVERTEX), 0, D3DFVF_CUSTOMVERTEX, D3DPOOL_DEFAULT, &g_pVB);

	VOID* pVertices;
	g_pVB->lpVtbl->Lock (g_pVB, 0, sizeof(g_Vertices), (BYTE**)&pVertices, 0);
	memcpy (pVertices, g_Vertices, sizeof(g_Vertices));
	g_pVB->lpVtbl->Unlock(g_pVB);

	color=D3DCOLOR_XRGB(63,127,127);

	D3DVECTOR vFrom, vTo, vUp;
	InitVector(vFrom,0.0f,0.0f,-3.0f);
	InitVector(vTo,0.0f,0.0f,0.0f);
	// we assume is Y co-or up !
	InitVector(vUp,0.0f,1.0f,0.0f);
	D3DXMATRIX matview, matproj;
	// use d3dx8 again
	D3DXMatrixLookAtLH( &matview, &vFrom, &vTo, &vUp );
	g_pd3dDevice->lpVtbl->SetTransform( g_pd3dDevice, D3DTS_VIEW, &matview );
	// use d3dx8 1 more time
	D3DXMatrixPerspectiveFovLH( &matproj, 0.785f, 1.33f, 0.01f, 20.0f );
	g_pd3dDevice->lpVtbl->SetTransform( g_pd3dDevice, D3DTS_PROJECTION, &matproj );

	// set the renderstate here, for global use.
	g_pd3dDevice->lpVtbl->SetTextureStageState(g_pd3dDevice, 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
	g_pd3dDevice->lpVtbl->SetTextureStageState(g_pd3dDevice, 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
	g_pd3dDevice->lpVtbl->SetTextureStageState(g_pd3dDevice, 0, D3DTSS_MIPFILTER, D3DTEXF_LINEAR );
	g_pd3dDevice->lpVtbl->SetRenderState(g_pd3dDevice, D3DRS_CULLMODE, D3DCULL_NONE);
	g_pd3dDevice->lpVtbl->SetRenderState(g_pd3dDevice, D3DRS_LIGHTING, FALSE);
	g_pd3dDevice->lpVtbl->SetRenderState(g_pd3dDevice, D3DRS_ALPHABLENDENABLE, TRUE);
	g_pd3dDevice->lpVtbl->SetRenderState(g_pd3dDevice, D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	g_pd3dDevice->lpVtbl->SetRenderState(g_pd3dDevice, D3DRS_DESTBLEND, D3DBLEND_INVDESTALPHA);
	g_pd3dDevice->lpVtbl->SetRenderState(g_pd3dDevice, D3DRS_ALPHAFUNC, D3DCMP_GREATEREQUAL);
	g_pd3dDevice->lpVtbl->SetRenderState(g_pd3dDevice, D3DRS_ALPHAREF, 0x7f);
	g_pd3dDevice->lpVtbl->SetRenderState(g_pd3dDevice, D3DRS_ALPHATESTENABLE, TRUE);

	return(1);
}

// Game Shutdown module here ------------------------------

int Game_Shutdown(void)
{
	DX8RELEASE(pTex);
	DX8RELEASE(g_pVB);
	DX8RELEASE(g_pD3D);
	DX8RELEASE(g_pd3dDevice);
	return(1);
}

// Game Main-Loop module here -----------------------------

int Game_Main(void)
{
	static float ry=0.0f;
	g_pd3dDevice->lpVtbl->Clear (g_pd3dDevice, 0, NULL, D3DCLEAR_TARGET, color, 1.0f, 0);
	g_pd3dDevice->lpVtbl->BeginScene (g_pd3dDevice);

	ry+=0.002f;
	if (ry>6.2832) ry=0.0f;
	D3DXMATRIX rotMat;
	// use d3dx8 1 last time !!
	D3DXMatrixRotationY(&rotMat, ry);
	g_pd3dDevice->lpVtbl->SetTransform( g_pd3dDevice, D3DTS_WORLD, &rotMat );

	g_pd3dDevice->lpVtbl->SetTexture(g_pd3dDevice, 0, pTex);

	g_pd3dDevice->lpVtbl->SetVertexShader (g_pd3dDevice, D3DFVF_CUSTOMVERTEX);
    g_pd3dDevice->lpVtbl->SetStreamSource (g_pd3dDevice, 0, g_pVB, sizeof(CUSTOMVERTEX));
    g_pd3dDevice->lpVtbl->DrawPrimitive (g_pd3dDevice, D3DPT_TRIANGLESTRIP, 0, 2);

	g_pd3dDevice->lpVtbl->SetTexture(g_pd3dDevice, 0, NULL);

	g_pd3dDevice->lpVtbl->EndScene (g_pd3dDevice);
	g_pd3dDevice->lpVtbl->Present (g_pd3dDevice, NULL, NULL, NULL, NULL);

	// Take a Screen Shot using DirectX8.1 new function.
	// Yup, function which newly provided in D3DX utility library
	// the following routine originally from Keith Newton, i made some changes,
	// please read the original article in GameDev.net.
	// http://www.gamedev.net/reference/articles/article1844.asp
	if (KEY_DOWN('P'))
	{
		HRESULT hr;
		IDirect3DSurface8* screenbuf=NULL;
		hr = g_pd3dDevice->lpVtbl->CreateImageSurface(g_pd3dDevice,WINDOW_WIDTH,WINDOW_HEIGHT,D3DFMT_A8R8G8B8,&screenbuf);
		if (hr!=D3D_OK)
		{
			screenbuf=NULL;
			MessageBox(g_hWnd,"CreateImageSurface failed","Error",MB_OK);
		}
		hr = g_pd3dDevice->lpVtbl->GetBackBuffer(g_pd3dDevice,0,D3DBACKBUFFER_TYPE_MONO,&screenbuf);
		if(hr != D3D_OK)
		{
			screenbuf->lpVtbl->Release(screenbuf);
			screenbuf=NULL;
			MessageBox(g_hWnd,"GetBackBuffer failed","Error",MB_OK);
		}
		D3DXSaveSurfaceToFile(SCRSHOT_NAME, D3DXIFF_BMP, screenbuf, NULL, NULL);
		screenbuf->lpVtbl->Release(screenbuf);
		MessageBox(g_hWnd,"Screen Shot taken !","Prompt",MB_OK);
	}
	//if (keystate[DIK_ESCAPE])
	if (KEY_DOWN(VK_ESCAPE))
		PostMessage(g_hWnd,WM_DESTROY,0,0);

	return(1);
}

// Other game logic modules start here ----------------------------

